/*

simple shaper

arguments: map_min map_max

*/

sketch.default2d();
var map_min = 0.0;
var map_max = 127.0;

var set_min = 10.0;
var set_max = 99.0;
var set_curve = 1;

var bg_rgba = [0.9,0.9,0.9,1.0];
var fg_rgba = [0.0,0.0,0.0,1.0];

// process arguments
if (jsarguments.length>1)
	map_min = jsarguments[1];
if (jsarguments.length>2)
	map_max = jsarguments[2];

draw();

function DataMapper(_min, _max, curve) {
	this.in_min = this.out_min = _min || 0;
	this.in_max = this.out_max = _max || 127;
	this.curve = curve || 1;
	
	this.SetInMin = function(in_min) {
		this.in_min = in_min;
	}
	this.SetInMax = function(in_max) {
		this.in_max = in_max;
	}
	this.SetOutMin = function(out_min) {
		this.out_min = out_min;
	}
	this.SetOutMax = function(out_max) {
		this.out_max = out_max;
	}
	this.SetCurve = function(curve) {
		this.curve = curve;
	}
	this.Map = function(val) {
		return ( Math.pow( ( ( val - this.in_min ) / (this.in_max - this.in_min) ), this.curve ) *
			(this.out_max - this.out_min) ) + this.out_min;
	}
}

function draw()
{
	var orig = sketch.screentoworld(0,0);
	
	with (sketch) {
		
		inset = 0.1;
		
		// Effects drawing quality
		shapeslice(180,1);
		
		// Erase background
		glclearcolor(bg_rgba[0],bg_rgba[1],bg_rgba[2],bg_rgba[3]);
		glclear();
		
		// Make a data mapper to make drawing easier
		sm = new DataMapper(map_min, map_max, 1.0);
		sm.SetOutMin( -(orig[1]-inset) );
		sm.SetOutMax( orig[1]-inset );
		
		xm = new DataMapper(map_min, map_max, 1.0);
		xm.SetOutMin( orig[0]+inset );
		xm.SetOutMax( -(orig[0]+inset) );
		ym = new DataMapper(map_min, map_max, 1.0);
		ym.SetOutMin( sm.Map(set_min) );
		ym.SetOutMax( sm.Map(set_max) );
		ym.SetCurve(set_curve);
		
		// Setup the pen attributes
		glcolor(fg_rgba);
		gllinewidth(1);
		
		// Figure out the increment
		inc = ( map_max - map_min ) / 150.0;
		
		// Graph the curve
		first = true;
		for( i=map_min, first=true; i<=map_max; i+=inc ) {
			if( first ) {
				moveto( xm.Map(i), ym.Map(i) );
				circle(0.05);
				first = false;
			}
			lineto( xm.Map(i), ym.Map(i) );
		}
		circle(0.05);
	}
}

function bang()
{
	draw();
	refresh();
	outlet(0,["params", set_min, set_max, set_curve]);
}

function msg_float(v)
{
	val = Math.min(Math.max(0,v),1);
	notifyclients();
	bang();
}

function setmin(v)
{
	set_min = Math.min(Math.max(map_min,v),map_max);
	notifyclients();
	draw();
	refresh();
}

function setmax(v)
{
	set_max = Math.min(Math.max(map_min,v),map_max);
	notifyclients();
	draw();
	refresh();
}

function setcurve(v)
{
	set_curve = Math.min(Math.max(0.02,v),20);
	notifyclients();
	draw();
	refresh();
}

function fsaa(v)
{
	sketch.fsaa = v;
	bang();
}

function bg(r,g,b,a)
{
	bg_rgba[0] = r/255.;
	bg_rgba[1] = g/255.;
	bg_rgba[2] = b/255.;
	bg_rgba[3] = a/255.;
	draw();
	refresh();
}

function fg(r,g,b,a)
{
	fg_rgba[0] = r/255.;
	fg_rgba[1] = g/255.;
	fg_rgba[2] = b/255.;
	fg_rgba[3] = a/255.;
	draw();
	refresh();
}

/*
function setvalueof(v)
{
	msg_float(v);
}

function getvalueof()
{
	return val;
}*/

// all mouse events are of the form: 
// onevent <x>, <y>, <button down>, <cmd(PC ctrl)>, <shift>, <capslock>, <option>, <ctrl(PC rbutton)>
// if you don't care about the additonal modifiers args, you can simply leave them out.
// one potentially confusing thing is that mouse events are in absolute screen coordinates, 
// with (0,0) as left top, and (width,height) as right, bottom, while drawing 
// coordinates are in relative world coordinates, with (0,0) as the center, +1 top, -1 bottom,
// and x coordinates using a uniform scale based on the y coordinates. to convert between screen 
// and world coordinates, use sketch.screentoworld(x,y) and sketch.worldtoscreen(x,y,z).

function onclick(x,y,but,cmd,shift,capslock,option,ctrl)
{
	// Figure out how wide the two cutters are
	gutter_width = sketch.worldtoscreen(0.2,0)[0] -
		sketch.worldtoscreen(0,0)[0];

	// cache mouse position for tracking delta movements
	last_x = x;
	last_y = y;
		
	if( x < gutter_width ) {
		control_zone = 0;
	} else if( x < ( box.rect[2] - box.rect[0] ) - gutter_width ) {
		control_zone = 1;
	} else {
		control_zone = 2;
	}
}
onclick.local = 1; //private. could be left public to permit "synthetic" events

function ondrag(x,y,but,cmd,shift,capslock,option,ctrl)
{
	var dy;

	// calculate delta movements
	dy = y - last_y;
	sscale = 1;
	if( shift ) {
		sscale *= 0.1;
	}
	
	// Min Zone 
	if( control_zone == 0 ) {
		setmin( set_min - dy * sscale );
		
	// Curve zone (reverses when min and max flip)
	} else if( control_zone == 1 ) {
		if( set_min < set_max ) {
			setcurve( set_curve + dy * (set_curve * 0.1) * sscale );
		} else {
			setcurve( set_curve - dy * (set_curve * 0.1) * sscale );
		}
		
	// Max Zone
	} else if( control_zone == 2 ) {
		setmax( set_max - dy * sscale );
	}
	
	// cache mouse position for tracking delta movements
	last_x = x;
	last_y = y;
}
ondrag.local = 1; //private. could be left public to permit "synthetic" events

function ondblclick(x,y,but,cmd,shift,capslock,option,ctrl)
{
	onclick(x,y,but,cmd,shift,capslock,option,ctrl);
	
	// Reset the proper zone based on the double click location
	if( control_zone == 0 ) {
		setmin( map_min );
    } else if( control_zone == 1 ) {
	    setcurve( 1 );
	} else if( control_zone == 2 ) {
		setmax( map_max );
	}
}
ondblclick.local = 1; //private. could be left public to permit "synthetic" events

function forcesize(w,h)
{
	if (w!=h) {
		h = w;
		box.size(w,h);
	}
}
forcesize.local = 1; //private

function onresize(w,h)
{
	draw();
	box.size(w,h)
	refresh();
}
onresize.local = 1; //private
